(*********************************************************)
(* TSLXProgressBar is a ProgressBar component for Delphi6*)
(* distributed for free, non-commercial purpose          *)
(*  2007 by Sl-Alex http://www.sl-alex.nm.ru/           *)
(*********************************************************)
unit SLXProgressBar;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, ExtCtrls, Graphics, JPEG;

type

//    
TSLXProgressDirection=(dtTopToBottom, dtBottomToTop,dtLeftToRight,dtRightToLeft);

//   - ,
TSLXFillStyle = (gsSolid,gsGradient,gsPicture);

//   - , 
TSLXDirection = (gdHorizontal,gdVertical);

//  
TSLXFill=class(TPersistent)
  private
    FOwner: TComponent;
    FStyle: TSLXFillStyle; //  
    FColor: TColor; //   
    FGradientStart: TColor; //    
    FGradientEnd: TColor; //    
    FGradientDirection: TSLXDirection; //   
    FPicture: TPicture; // ,    
    procedure FSetColor(NewColor: TColor); //    
    procedure FSetStyle(NewStyle: TSLXFillStyle); //   
    procedure FSetGradientStart(NewColor: TColor); //     
    procedure FSetGradientEnd(NewColor: TColor); //     
    procedure FSetDirection(NewDirection: TSLXDirection); //    
    procedure FSetPicture(NewPicture: TPicture); //  
    procedure FGradientFill(NewCanvas: TCanvas; NewRect: TRect; NewBeginColor, NewEndColor: TColor; NewDirection: TSLXDirection); //   
  public
    constructor Create(AOwner: TComponent); //  
    destructor Destroy; override; //  
  published
    procedure FillCanvas(const X1,Y1,X2,Y2: Integer); //  ( )
    property Color: TColor read FColor write FSetColor; //   
    property Style: TSLXFillStyle read FStyle write FSetStyle;//   (/)
    property GradientDirection: TSLXDirection read FGradientDirection write FSetDirection; //  
    property GradientStart: TColor read FGradientStart write FSetGradientStart; //   
    property GradientEnd: TColor read FGradientEnd write FSetGradientEnd;//   
    property Picture: TPicture read FPicture write FSetPicture; // ,    
end;

//  
TSLXProgressBar = class(TCustomPanel)
  private
    FMin: Double;   //  
    FMax: Double;   //  
    FValue: Double; //  
    FBar: TSLXFill; //  
    FProgressBitMap: TBitMap; //   ,        
    FDirection: TSLXProgressDirection; //    
    FFont: TFont; // ,   
    FShowText: Boolean; //   
    FOnPaint: TNotifyEvent; //  
    FColor: TColor; //  
    procedure FSetMin(NewValue: Double); //   
    procedure FSetMax(NewValue: Double); //   
    procedure FSetBar(NewValue: TSLXFill); //   
    procedure FSetValue(NewValue: Double); //   
    procedure FSetDirection(NewValue: TSLXProgressDirection); //     
    procedure FPaint(AlsoScreen: Boolean); //    
    procedure FFontChange(Sender: TObject); //  
    procedure FSetFont(NewValue: TFont); //  
    procedure FSetShowText(NewValue: Boolean); //   
    procedure FSetColor(NewValue: TColor); //   
  protected
    { Protected declarations }
  public
    //    
    property DockManager;
    property Canvas;
    //--
    constructor Create(AOwner: TComponent); override; //  
    destructor Destroy; override; //  
    procedure Paint; override; //  
    property BitMap: TBitMap read FProgressBitMap write FProgressBitMap; // ,    
  published
    //    
    property Align;
    property Alignment;
    property Anchors;
    property BorderWidth;
    property Constraints;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnContextPopup;
    property OnDockDrop;
    property OnDockOver;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDock;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnResize;
    property OnStartDock;
    property OnStartDrag;
    property OnUnDock;
    property ShowHint;
    //--
    property BackGround: TColor read FColor write FSetColor; //  
    property ShowText: Boolean read FShowText write FSetShowText; //   
    property Min: Double read FMin write FSetMin; //  
    property Max: Double read FMax write FSetMax; //  
    property OnPaint: TNotifyEvent read FOnPaint write FOnPaint; //   
    property Bar: TSLXFill read FBar write FSetBar; //  
    property Direction: TSLXProgressDirection read FDirection write FSetDirection;//  
    property Value: Double read FValue write FSetValue; //  
    property Font: TFont read FFont write FSetFont; // ,   
end;

//  
procedure Register;

implementation

{$R SLXProgressBar.dcr}

//    
procedure CopyFontProps(Source: TFont; Dest: TFont);
begin
  Dest.Charset:=Source.Charset;
  Dest.Color:=Source.Color;
  Dest.Height:=Source.Height;
  Dest.Name:=Source.Name;
  Dest.Pitch:=Source.Pitch;
  Dest.Size:=Source.Size;
  Dest.Style:=Source.Style;
end;

//   
procedure TSLXProgressBar.FSetColor(NewValue: TColor);
begin
  FColor:=NewValue;
  FPaint(True);
end;

//   
procedure TSLXProgressBar.FSetShowText(NewValue: Boolean);
begin
  FShowText:=NewValue;
  FPaint(True);
end;

//  
procedure TSLXProgressBar.FFontChange(Sender: TObject);
begin
  CopyFontProps(Sender as TFont,FFont);
  FPaint(True);
end;

//  
procedure TSLXProgressBar.FSetFont(NewValue: TFont);
begin
  CopyFontProps(NewValue,FFont);
  FPaint(True);
end;

//     
procedure TSLXProgressBar.FSetDirection(NewValue: TSLXProgressDirection);
begin
  FDirection:=NewValue;
  FPaint(True);
end;

//   
procedure TSLXProgressBar.FSetBar(NewValue: TSLXFill);
begin
  FBar:=NewValue;
  FPaint(True);
end;

//    
procedure TSLXProgressBar.FPaint(AlsoScreen: Boolean);
var i, X1,Y1,X2,Y2: Integer;
    CapWidth,CapHeight: Integer;
begin
  FProgressBitMap.Width:=Width;
  FProgressBitMap.Height:=Height;
  X1:=0; Y1:=0; X2:=0; Y2:=0;
  if FDirection=dtLeftToRight then begin
    X1:=0;
    Y1:=0;
    X2:=Round(Width*(FValue-FMin)/(FMax-FMin));
    Y2:=Height;
  end;
  if FDirection=dtRightToLeft then begin
    X1:=Width-Round(Width*(FValue-FMin)/(FMax-FMin));
    Y1:=0;
    X2:=Width;
    Y2:=Height;
  end;
  if FDirection=dtTopToBottom then begin
    X1:=0;
    Y1:=0;
    X2:=Width;
    Y2:=Round(Height*(FValue-FMin)/(FMax-FMin));
  end;
  if FDirection=dtBottomToTop then begin
    X1:=0;
    Y1:=Height-Round(Height*(FValue-FMin)/(FMax-FMin));
    X2:=Width;
    Y2:=Height;
  end;
  FProgressBitMap.Canvas.Brush.Color:=FColor;
  FProgressBitMap.Canvas.FillRect(Rect(0,0,Width,Height));
  FBar.FillCanvas(X1,Y1,X2,Y2);
  FProgressBitMap.Canvas.Pen.Color:=clBlack;
  for i:=0 to BorderWidth-1 do begin
    FProgressBitMap.Canvas.MoveTo(i,i);
    FProgressBitMap.Canvas.LineTo(i,Height-1-i);
    FProgressBitMap.Canvas.LineTo(Width-1-i,Height-1-i);
    FProgressBitMap.Canvas.LineTo(Width-1-i,i);
    FProgressBitMap.Canvas.LineTo(i,i);
  end;
  Caption:=Format('%.2f',[FValue]);
  if FShowText then begin
    CopyFontProps(FFont,FProgressBitMap.Canvas.Font);
    CapWidth:=FProgressBitMap.Canvas.TextWidth(Caption);
    CapHeight:=FProgressBitMap.Canvas.TextHeight(Caption);
    SetBkMode(FProgressBitMap.Canvas.Handle, TRANSPARENT);
    FProgressBitMap.Canvas.TextOut(Width div 2 - CapWidth div 2,Height div 2 - CapHeight div 2,Caption);
    SetBkMode(FProgressBitMap.Canvas.Handle, OPAQUE);
  end;
  if AlsoScreen then RePaint;
end;

//  
procedure TSLXProgressBar.Paint;
begin
  bitblt(Canvas.Handle, 0, 0, width, height, FProgressBitMap.Canvas.Handle, 0, 0, SRCCOPY);
  if Assigned(OnPaint) then OnPaint(Self);
end;

//   
procedure TSLXProgressBar.FSetMin(NewValue: Double);
begin
  FMin:=NewValue;
  FPaint(True);
end;

//   
procedure TSLXProgressBar.FSetMax(NewValue: Double);
begin
  FMax:=NewValue;
  FPaint(True);
end;

//   
procedure TSLXProgressBar.FSetValue(NewValue: Double);
begin
  FValue:=NewValue;
  FPaint(True);
end;

//  
constructor TSLXProgressBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FBar:=TSLXFill.Create(Self);
  // ,    
  FProgressBitMap:=TBitMap.Create;
  FFont:=TFont.Create;
  FFont.OnChange:=FFontChange;
  FShowText:=True;
  FDirection:=dtLeftToRight;
  BorderWidth:=1;
  DoubleBuffered:=True;
  FColor:=clSkyBlue;
  FMin:=0;
  FMax:=1;
  FValue:=0.7;
  Width:=161;
  Height:=17;
  FPaint(False);
end;

//  
destructor TSLXProgressBar.Destroy;
begin
  FFont.Destroy;
  FProgressBitMap.Destroy;
  FBar.Destroy;
  inherited Destroy;
end;

{------------------------------------------------------------------------------}
//   --------------------------------------------------------------
{------------------------------------------------------------------------------}

//  
procedure TSLXFill.FSetPicture(NewPicture: TPicture);
begin
  FPicture.Assign(NewPicture);
  (FOwner as TSLXProgressBar).FPaint(True);
end;

procedure TSLXFill.FillCanvas(const X1,Y1,X2,Y2: Integer);
begin
  //  
  if FStyle=gsGradient then begin
    FGradientFill((FOwner as TSLXProgressBar).BitMap.Canvas,Rect(X1,Y1,X2,Y2),FGradientStart,FGradientEnd,FGradientDirection);
  end;
  //  
  if FStyle=gsSolid then begin
    (FOwner as TSLXProgressBar).BitMap.Canvas.Brush.Color:=FColor;
    (FOwner as TSLXProgressBar).BitMap.Canvas.FillRect(Rect(X1,Y1,X2,Y2));
  end;
  //  
  if FStyle=gsPicture then begin
    (FOwner as TSLXProgressBar).BitMap.Canvas.StretchDraw(Rect(X1,Y1,X2,Y2),FPicture.Graphic);
  end;
end;

//   
procedure TSLXFill.FGradientFill(NewCanvas: TCanvas; NewRect: TRect; NewBeginColor, NewEndColor: TColor; NewDirection: TSLXDirection);
var
  i, j, h, w, NewStepsCount: Integer;
  R, G, B: Longword;
  beginRGBvalue, RGBdifference: array[0..2] of LongInt;
  IntRect: TRect;
begin
  NewStepsCount:=1;
  beginRGBvalue[0] := GetRvalue(colortoRGB(NewBeginColor));
  beginRGBvalue[1] := GetGvalue(colortoRGB(NewBeginColor));
  beginRGBvalue[2] := GetBvalue(colortoRGB(NewBeginColor));

  RGBdifference[0] := GetRvalue(colortoRGB(NewEndColor)) - beginRGBvalue[0];
  RGBdifference[1] := GetGvalue(colortoRGB(NewEndColor)) - beginRGBvalue[1];
  RGBdifference[2] := GetBvalue(colortoRGB(NewEndColor)) - beginRGBvalue[2];

  NewCanvas.pen.Style := pssolid;
  NewCanvas.pen.mode := pmcopy;
  j := 0;

  h := NewRect.Bottom - NewRect.Top;
  w := NewRect.Right - NewRect.Left;
  IntRect.Left:=NewRect.Left;
  IntRect.Top:=NewRect.Top;
  IntRect.Right:=NewRect.Right;
  IntRect.Bottom:=NewRect.Bottom;

  R:=0;
  G:=0;
  B:=0;
  if NewDirection=gdHorizontal then NewStepsCount:=w;
  if NewDirection=gdVertical   then NewStepsCount:=h;
  for i := 0 to NewStepsCount do begin
    if NewDirection=gdHorizontal then begin
      IntRect.Left  := NewRect.Left+muldiv(i - 1, w, NewStepsCount);
      IntRect.Right := NewRect.Left+muldiv(i, w, NewStepsCount);
    end;
    if NewDirection=gdVertical then begin
      IntRect.Top  :=   NewRect.Top+muldiv(i - 1, h, NewStepsCount);
      IntRect.Bottom := NewRect.Top+muldiv(i, h, NewStepsCount);
    end;
    if NewStepsCount>0 then begin
      R := beginRGBvalue[0] + muldiv(j, RGBDifference[0], NewStepsCount);
      G := beginRGBvalue[1] + muldiv(j, RGBDifference[1], NewStepsCount);
      B := beginRGBvalue[2] + muldiv(j, RGBDifference[2], NewStepsCount);
    end;
    NewCanvas.Brush.Color := RGB(R, G, B);
    if NewDirection=gdHorizontal then begin
      patBlt(NewCanvas.Handle, IntRect.Left, NewRect.Top, IntRect.Right - IntRect.Left, h, patcopy);
    end;
    if NewDirection=gdVertical then begin
      patBlt(NewCanvas.Handle, NewRect.Left, IntRect.Top, w, IntRect.Bottom - IntRect.Top, patcopy);
    end;
    Inc(j);
  end;
end;

//    
procedure TSLXFill.FSetColor(NewColor: TColor);
//    ,   gsSolid,   
begin
  FColor:=NewColor;
  if FStyle=gsSolid then (FOwner as TSLXProgressBar).FPaint(True);
end;

//   
procedure TSLXFill.FSetStyle(NewStyle: TSLXFillStyle);
begin
  FStyle:=NewStyle;
  (FOwner as TSLXProgressBar).FPaint(True);
end;

//     
procedure TSLXFill.FSetGradientStart(NewColor: TColor);
//     ,   gsGradient,   
begin
  FGradientStart:=NewColor;
  if FStyle=gsGradient then (FOwner as TSLXProgressBar).FPaint(True);
end;

//     
procedure TSLXFill.FSetGradientEnd(NewColor: TColor);
//     ,   gsGradient,   
begin
  FGradientEnd:=NewColor;
  if FStyle=gsGradient then (FOwner as TSLXProgressBar).FPaint(True);
end;

//    
procedure TSLXFill.FSetDirection(NewDirection: TSLXDirection);
//    ,   gsGradient,   
begin
  FGradientDirection:=NewDirection;
  if FStyle=gsGradient then (FOwner as TSLXProgressBar).FPaint(True);
end;

//  
constructor TSLXFill.Create(AOwner: TComponent);
begin
  inherited Create;
  FOwner:=AOwner;
  FStyle:=gsGradient;
  FColor:=clSilver;
  FGradientStart:=clCream;
  FGradientEnd:=clBlue;
  FGradientDirection:=gdVertical;
//  FGradientSteps:=17;
  FPicture:=TPicture.Create;
end;

//  
destructor TSLXFill.Destroy;
begin
  FPicture.Destroy;
  inherited Destroy;
end;

//  
procedure Register;
begin
  RegisterComponents('Sl-Alex', [TSLXProgressBar]);
end;

end.
